@@ -1,5 +1,6 @@
-;;; CLtL2-kompatible Definitionen
+;;; ANSI-compatible definitions
 ;;; Bruno Haible 21.7.1994
+;;; Sam Steingold 1999-2004
 
 ;; ============================================================================
 
@@ -326,6 +327,8 @@
            (SYSTEM::*PRIN-STREAM*       NIL) ; CLISP specific
            (SYSTEM::*PRIN-LINELENGTH*   79)  ; CLISP specific
            (SYSTEM::*PRIN-LINE-PREFIX*  NIL) ; CLISP specific
+           (CUSTOM::*PRINT-SYMBOLS-LONG* NIL) ; CLISP specific
+           (COMPILER::*LOAD-FORMS*      NIL) ; CLISP specific
            ;; reader variables:
            (*READ-BASE*                 10)
            (*READ-DEFAULT-FLOAT-FORMAT* 'SINGLE-FLOAT)
@@ -464,7 +467,10 @@
                (eq veltype 'CHARACTER))
            (apply #'write-char-sequence sequence stream rest))
           ((or (subtypep seltype 'INTEGER) (subtypep veltype 'INTEGER))
-           (apply #'write-byte-sequence sequence stream rest))
+           ;; `write-byte-sequence' accepts :NO-HANG and returns an extra value
+           ;; since making `write-char-sequence' do the same would be hairy,
+           ;; we ignore this second return value for the sake of ANSI
+           (values (apply #'write-byte-sequence sequence stream rest)))
           (t
            (error (TEXT "~S: element types of ~S and ~S are ambiguous. Please use ~S or ~S.")
                   'write-sequence sequence stream
