@@ -6,9 +6,11 @@
     `(BLOCK ,b
        (LET ((*ERROR-HANDLER*
               #'(LAMBDA (&REST ARGS)
-                  (LET ((*PRINT-READABLY* NIL) (*PRINT-CIRCLE* NIL))
-                    (TERPRI) (APPLY #'FORMAT T (CDR ARGS)))
-                  (RETURN-FROM ,b 'ERROR))))
+                  (with-standard-io-syntax
+                    (let* ((*print-readably* nil)
+                           (error-message (apply #'format nil (cdr args))))
+                      (terpri) (princ error-message)
+                      (return-from ,b (values 'error error-message)))))))
          ,@forms))))
 
 #+(or AKCL ECL)
@@ -50,15 +52,15 @@
 (defun do-test (stream log &optional (ignore-errors t))
   (let ((eof "EOF") (error-count 0) (total-count 0))
     (loop
-      (incf total-count)
       (let ((form (read stream nil eof))
             (result (read stream nil eof)))
         (when (or (eq form eof) (eq result eof)) (return))
+        (incf total-count)
         (print form)
-        (let ((my-result
-               (if ignore-errors
-                  (with-ignored-errors (eval form)) ; return ERROR on errors
-                  (eval form)))) ; don't disturb the condition system when testing it!
+        (multiple-value-bind (my-result error-message)
+            (if ignore-errors
+                (with-ignored-errors (eval form)) ; return ERROR on errors
+                (eval form)) ; don't disturb the condition system when testing it!
           (cond ((eql result my-result)
                  (format t "~%EQL-OK: ~S" result))
                 ((equal result my-result)
@@ -68,19 +70,19 @@
                 (t
                  (incf error-count)
                  (format t "~%ERROR!! ~S should be ~S !" my-result result)
-                 (format log "~%Form: ~S~%CORRECT: ~S~%~7A: ~S~%"
+                 (format log "~%Form: ~S~%CORRECT: ~S~%~7A: ~S~%~@[~A~%~]"
                              form result lisp-implementation-type
-                             my-result))))))
+                             my-result error-message))))))
     (values total-count error-count)))
 
 (defun do-errcheck (stream log &optional ignore-errors)
   (declare (ignore ignore-errors))
   (let ((eof "EOF") (error-count 0) (total-count 0))
     (loop
-      (incf total-count)
       (let ((form (read stream nil eof))
             (errtype (read stream nil eof)))
         (when (or (eq form eof) (eq errtype eof)) (return))
+        (incf total-count)
         (print form)
         (let ((my-result (nth-value 1 (ignore-errors (eval form)))))
           (multiple-value-bind (typep-result typep-error)
@@ -95,8 +97,12 @@
                                my-result)))))))
     (values total-count error-count)))
 
+(defvar *run-test-tester* #'do-test)
+(defvar *run-test-ignore-errors* t)
+
 (defun run-test (testname
-                 &optional (tester #'do-test) (ignore-errors t)
+                 &optional (tester *run-test-tester*)
+                           (ignore-errors *run-test-ignore-errors*)
                  &aux (logname (merge-extension "erg" testname))
                       error-count total-count)
   (with-open-file (s (merge-extension "tst" testname) :direction :input)
@@ -118,6 +124,24 @@
        (incf ,error-count ,err)
        (incf ,total-count ,tot))))
 
+(defun run-files (files)
+  (let ((error-count 0) (total-count 0))
+    (dolist (file files)
+      (with-accumulating-errors (error-count total-count)
+        (run-test file)))
+    (format
+     t "~&~s: finished ~:d file~:p (~:d error~:p out of ~:d test~:p)~%~S~%"
+     'run-files (length files) error-count total-count files)
+    (values error-count total-count)))
+
+(defun run-some-tests (&optional (dirlist '("./")))
+  (let ((files (mapcan (lambda (dir)
+                         (directory (make-pathname :name :wild :type "tst"
+                                                   :defaults dir)))
+                       dirlist)))
+    (if files (run-files files)
+        (warn "no TST files in directories ~S" dirlist))))
+
 (defun run-all-tests (&optional (disable-risky t))
   (let ((error-count 0) (total-count 0)
         (*features* (if disable-risky *features*
@@ -128,7 +152,7 @@
                   #+CLISP           "bin-io"
                   #-AKCL            "characters"
                   #+(or CLISP ALLEGRO CMU) "clos"
-                  #+CLISP ,@(unless disable-risky "defhash")
+                  #+CLISP ,@(unless disable-risky '("defhash"))
                   #+(and CLISP UNICODE) "encoding"
                                     "eval20"
                   #+(and CLISP FFI) "ffi"
@@ -137,7 +161,7 @@
                   #+CLISP           "genstream"
                   #+XCL             "hash"
                                     "hashlong"
-                  #+CLISP ,@(unless disable-risky "hashweak")
+                  #+CLISP ,@(unless disable-risky '("hashweak"))
                                     "iofkts"
                                     "lambda"
                                     "lists151"
@@ -165,7 +189,7 @@
                   #+XCL             "tprint"
                   #+XCL             "tread"
                                     "type"
-                  #+CLISP ,@(unless disable-risky "weakptr")))
+                  #+CLISP ,@(unless disable-risky '("weakptr"))))
       (with-accumulating-errors (error-count total-count) (run-test ff)))
     #+(or CLISP ALLEGRO CMU)
     (with-accumulating-errors (error-count total-count)
